/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file Slider.hpp
/// @desc Slider widget class header.
/// @edit 2024-10-31


#pragma once
#include "Button.hpp"
#include "Rect.hpp"
#include "JUI/Mixins/Draggable.hpp"

namespace JUI
{

    template<typename T>
    T roundMultiple( T value, T multiple )
    {
        if (multiple == 0) return value;
        return static_cast<T>(std::round(static_cast<double>(value)/static_cast<double>(multiple))*static_cast<double>(multiple));
    }

    /// A slider is a widget with a handle which can be pulled back and forth to change the value.
    class Slider : public Rect, public Clickable, public Hoverable
    {
    public:
        Event<float> ValueChanged;

        Slider();
        explicit Slider(JUI::Widget* parent);

        [[nodiscard]] float Minimum() const;
        [[nodiscard]] float Maximum() const;
        [[nodiscard]] float Interval() const;
        [[nodiscard]] float CurrentValue() const;
        [[nodiscard]] Color4 ScrubberColor() const;
        [[nodiscard]] float ScrubberWidth() const;
        [[nodiscard]] bool Dragging() const;
        [[nodiscard]] float Range() const;

        void Minimum(float min);
        void Maximum(float max);
        void Interval(float inter);
        void CurrentValue(float value);
        void ScrubberColor(const Color4& color);
        void ScrubberWidth(float width);
        void SetDragging(bool value);



        void OnClick(const J3ML::LinearAlgebra::Vector2 &MousePos, const JUI::MouseButton &MouseButton) override;
        void OnRelease(const J3ML::LinearAlgebra::Vector2 &MousePos, const JUI::MouseButton &MouseButton, bool MouseStillOver) override;
        void OnHover(const J3ML::LinearAlgebra::Vector2 &MousePos) override;
        void OnExit(const J3ML::LinearAlgebra::Vector2 &MousePos) override;
        void Update(float delta) override;
        void Draw() override;

    protected:
        float minimum = 0;
        float maximum = 1;
        float interval = 0.1;
        float current;
        bool dragging = false;
        float scrubber_width = 20;
        Color4 scrubber_color = Colors::White;
    private:
    };
}
